package com.sophiaatkinson.pronouns;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.bukkit.entity.Player;

public class PronounsCommand implements CommandExecutor, TabCompleter {

    private final PronounsPlugin plugin;

    public PronounsCommand(PronounsPlugin plugin) {
        this.plugin = plugin;
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        if (!(sender instanceof Player player)) {
            sender.sendMessage(ChatColor.RED + "Only players can use this command.");
            return true;
        }

        if (args.length == 0) {
            player.sendMessage(ChatColor.YELLOW + "Usage: /pronouns help");
            return true;
        }

        String subcommand = args[0].toLowerCase();

        switch (subcommand) {
            case "set" -> {
                if (args.length < 2) {
                    player.sendMessage(ChatColor.RED + "Please provide your pronouns. Example: /pronouns set she/her");
                    return true;
                }
                String pronouns = String.join(" ", Arrays.copyOfRange(args, 1, args.length));

                for (String blocked : plugin.getConfig().getStringList("blocked-pronouns")) {
                    if (pronouns.contains(blocked)) {
                        player.sendMessage(ChatColor.RED + "Your pronouns contain inappropriate language and cannot be set.");
                        return true;
                    }
                }

                plugin.setPronouns(player.getUniqueId(), pronouns);
                player.sendMessage(ChatColor.GREEN + "Your pronouns have been set to: " + ChatColor.AQUA + pronouns);
            }

            case "view" -> {
                if (args.length == 1) {
                    String pronouns = plugin.getPronouns(player.getUniqueId());
                    if (pronouns != null) {
                        player.sendMessage(ChatColor.GREEN + "Your pronouns are: " + ChatColor.AQUA + pronouns);
                    } else {
                        player.sendMessage(ChatColor.YELLOW + "You haven't set your pronouns yet.");
                    }
                } else {
                    Player target = Bukkit.getPlayerExact(args[1]);
                    if (target != null) {
                        String pronouns = plugin.getPronouns(target.getUniqueId());
                        if (pronouns != null) {
                            player.sendMessage(ChatColor.GREEN + target.getName() + "'s pronouns are: " + ChatColor.AQUA + pronouns);
                        } else {
                            player.sendMessage(ChatColor.YELLOW + target.getName() + " hasn't set pronouns.");
                        }
                    } else {
                        player.sendMessage(ChatColor.RED + "Player not found.");
                    }
                }
            }

            case "reset" -> {
                plugin.setPronouns(player.getUniqueId(), null);
                player.sendMessage(ChatColor.GREEN + "Your pronouns have been reset.");
            }

            case "help" -> {
                player.sendMessage(ChatColor.AQUA + "=== Pronouns Plugin Help ===");
                player.sendMessage(ChatColor.YELLOW + "/pronouns set <pronouns> " + ChatColor.WHITE + "- Set your pronouns");
                player.sendMessage(ChatColor.YELLOW + "/pronouns view <player> " + ChatColor.WHITE + "- View your own or another player's pronouns");
                player.sendMessage(ChatColor.YELLOW + "/pronouns reset " + ChatColor.WHITE + "- Reset your pronouns");
                player.sendMessage(ChatColor.YELLOW + "/pronouns version " + ChatColor.WHITE + "- Show plugin version info");
                player.sendMessage(ChatColor.YELLOW + "/pronouns debug " + ChatColor.WHITE + "- Generate debug file");
                player.sendMessage(ChatColor.YELLOW + "/pronouns admin " + ChatColor.WHITE + "- Admin tools (list, set, remove)");
            }

            case "version" -> {
                player.sendMessage(ChatColor.AQUA + "=== Version Info ===");
                player.sendMessage(ChatColor.YELLOW + "PronounsPlaceholder: " + ChatColor.WHITE + plugin.getDescription().getVersion());
                player.sendMessage(ChatColor.YELLOW + "Server Version: " + ChatColor.WHITE + Bukkit.getVersion());
            }

            case "debug" -> {
                plugin.generateDebugFile();
                player.sendMessage(ChatColor.GREEN + "Debug file written to plugin folder as debug.txt");
            }

            case "admin" -> {
                if (!player.hasPermission("pronouns.admin")) {
                    player.sendMessage(ChatColor.RED + "You don't have permission to use admin commands.");
                    return true;
                }

                if (args.length < 2) {
                    player.sendMessage(ChatColor.RED + "Usage: /pronouns admin <list|remove|set>");
                    return true;
                }

                switch (args[1].toLowerCase()) {
                    case "list" -> {
                        Map<UUID, String> all = plugin.getAllPronouns();
                        if (all.isEmpty()) {
                            player.sendMessage(ChatColor.YELLOW + "No pronouns have been set.");
                        } else {
                            player.sendMessage(ChatColor.AQUA + "=== All Player Pronouns ===");
                            for (Map.Entry<UUID, String> entry : all.entrySet()) {
                                OfflinePlayer p = Bukkit.getOfflinePlayer(entry.getKey());
                                player.sendMessage(ChatColor.YELLOW + p.getName() + ": " + ChatColor.AQUA + entry.getValue());
                            }
                        }
                    }

                    case "remove" -> {
                        if (args.length < 3) {
                            player.sendMessage(ChatColor.RED + "Usage: /pronouns admin remove <player>");
                            return true;
                        }
                        OfflinePlayer target = Bukkit.getOfflinePlayer(args[2]);
                        plugin.setPronouns(target.getUniqueId(), null);
                        player.sendMessage(ChatColor.GREEN + "Removed pronouns for " + target.getName());
                    }

                    case "set" -> {
                        if (args.length < 4) {
                            player.sendMessage(ChatColor.RED + "Usage: /pronouns admin set <player> <pronouns>");
                            return true;
                        }
                        OfflinePlayer target = Bukkit.getOfflinePlayer(args[2]);
                        String pronouns = String.join(" ", Arrays.copyOfRange(args, 3, args.length));
                        plugin.setPronouns(target.getUniqueId(), pronouns);
                        player.sendMessage(ChatColor.GREEN + "Set " + target.getName() + "'s pronouns to: " + ChatColor.AQUA + pronouns);
                    }

                    default -> player.sendMessage(ChatColor.RED + "Unknown admin subcommand.");
                }
            }

            default -> player.sendMessage(ChatColor.RED + "Unknown subcommand. Use /pronouns help for options.");
        }

        return true;
    }

    @Override
    public List<String> onTabComplete(CommandSender sender, Command command, String alias, String[] args) {
        if (args.length == 1) {
            return Arrays.asList("set", "view", "reset", "help", "version", "debug", "admin");
        }

        if (args.length == 2 && args[0].equalsIgnoreCase("view")) {
            List<String> names = new ArrayList<>();
            for (Player p : Bukkit.getOnlinePlayers()) {
                names.add(p.getName());
            }
            return names;
        }

        if (args.length == 2 && args[0].equalsIgnoreCase("set")) {
            return plugin.getPronounSuggestions();
        }
    
        if (args.length == 2 && args[0].equalsIgnoreCase("admin")) {
            return Arrays.asList("list", "remove", "set");
        }
    
        if (args.length == 3 && args[0].equalsIgnoreCase("admin")) {
            if (args[1].equalsIgnoreCase("remove") || args[1].equalsIgnoreCase("set")) {
                List<String> names = new ArrayList<>();
                for (OfflinePlayer p : Bukkit.getOfflinePlayers()) {
                    names.add(p.getName());
                }
                return names;
            }
        }
    
        if (args.length == 4 && args[0].equalsIgnoreCase("admin") && args[1].equalsIgnoreCase("set")) {
            return plugin.getPronounSuggestions();
        }
    
        return Collections.emptyList();
    }
}
